/*=========================================================================
  This header file is common for all TVIA SDK code and included in all C files.

  This file contains two parts:
    Part I - Memory access
    Part II - I/O access

   -!!!Read This Message!!!-

  MAKE SURE TO DEFINE THE FOLLOWING COMPILATION SWITCHES (DIRECTIVES) CORRECTLY
  IN FILE GLOBAL.H:
       1) LINEAR_MEMORY
  ========================================================================*/

#ifndef __TVIA_IGSIOMEM_H
#define __TVIA_IGSIOMEM_H
/*-----------------------------------------------------------------------------
     Part I - Memory access
  -----------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
   The 32 bits address of TVIA linear frame buffer is defined as follows:

       31 30 .. 25 24 23 22 21 ...............3 2 1 0
       |     | |     |       |
       |     | |     |_______________________|__ 4MB range (could be
       |     | |     video memory or
       |     | |     memory mapped I/O)
       |     | |
       |     | |________0: normal video memory access
       |     |        1: memory mapped I/O access
       |     |
       |____________|_________ Upper 8 bits of the 32 bits physical address

   After chip is enabled, the linear frame buffer can be figured out
   from register 3CE_35 which contains the upper 8 bits of the 32 bits
   physical address of the video linear frame buffer.
   For example,
     outp(0x3CE, 0x35);
     bRegValue = inp(0x3CF);
   Assume bRegValue = 0x??, then 0x??000000 is the beginning of video linear
   frame buffer physical address.

   This video memory physical address can also be obtained from PCI configuration
   register 0x10 (which value is identical to the one obtained from 3CE_35).

   If bit 23 of the 32 bit address is 1, then memory mapped I/O is enabled for
   that memory access cycle.
   For example,
    "*(0x??8003CE) = 0x35" is accessing register 3CE_35, not video memory.
 -----------------------------------------------------------------------------*/

#if LINEAR_MEMORY

  /* --- This is used to access 2D engine, Audio, and TV Encoder registers --- */
  #define bpTVAudio2DRegBase  (volatile BYTE_ *) (dwMEM_IO_BASE + 0x000B0000)

  /* --- This is used to access video linear frame buffer --- */
  #define bpVideoMemPtr       (BYTE_ *) dwLINEAR_BASE

#else /* use banking memory scheme */

  /*===================================================================
    The following pointers are DOS based value, e.g 0xB0000000 actually
    is 0xB000:0, or physically at 0xB0000, not physically at 0xB0000000.
    Similarly, 0xA0000000 is 0xA000:0.
    (The DOS compiler, Borland C++, makes the conversion)
   ====================================================================*/
  #define bpTVAudio2DRegBase  (volatile BYTE_ far_ *) 0xB0000000

  #define bpVideoMemPtr       (BYTE_ far_ *) 0xA0000000

#endif

/*defines for dwFlag in BitBlockTransfer()*/
#define SrcIsVideo    0
#define SrcIsSystem   1
#define DstIsVideo    0
#define DstIsSystem   2

#ifdef __cplusplus
extern "C" {
#endif

BYTE_ far_ *GetFrameBufferPtr(void);

void SetEnvironmentVariables(void);
void DrawPixel(WORD_ x, WORD_ y, DWORD_ dwData);
void DrawLine(WORD_ x0, WORD_ y0, WORD_ x1, WORD_ y1, DWORD_ dwData);
void FillRect(WORD_ x0, WORD_ y0, WORD_ x1, WORD_ y1, DWORD_ dwData);

void BitBlockTransfer(BYTE_ far_ *dwSrcPtr, BYTE_ far_ *dwDstPtr, DWORD_ dwSize, DWORD_ dwFlag);

#if LINEAR_MEMORY
#else
void SetBank(WORD_ iBankNum);
void SetWriteBank(WORD_ iBankNum);
void SetReadBank(WORD_ iBankNum);
void UnlockTVAudioReg(void);
void LockTVAudioReg(void);
void Unlock2DReg(void);
void Lock2DReg(void);
void EnableBankingMemory(void);
void DisableBankingMemory(void);
#endif /*LINEAR_MEMORY*/




/*-----------------------------------------------------------------------------
     Part II - I/O access
  -----------------------------------------------------------------------------*/

/* Note: R - readable, W - Writable, R/W - read/writable */
#define   PORT46E8      0x46E8 /* R   */
#define   PORT102       0x102  /* R/W */
#define   MISCREAD      0x3CC  /* R   */
#define   MISCWRITE     0x3C2  /* W   */
#define   SEQINDEX      0x3C4  /* R/W */
#define   SEQDATA       0x3C5  /* R/W */
#define   CRTINDEX      0x3D4  /* R/W */
#define   CRTDATA       0x3D5  /* R/W */
#define   ATTRRESET     0x3DA  /* R/W */
#define   ATTRINDEX     0x3C0  /* R/W */
#define   ATTRDATAW     0x3C0  /* W, Attrib write data port */
#define   ATTRDATAR     0x3C1  /* R, Attrib read data port  */
#define   GRAINDEX      0x3CE  /* R/W */
#define   GRADATA       0x3CF  /* R/W */
#define   RAMDACMASK    0x3C6  /* R/W, Mask register */
#define   RAMDACINDEXR  0x3C7  /* R/W, RAM read index port  */
#define   RAMDACINDEXW  0x3C8  /* R/W, RAM write index port */
#define   RAMDACDATA    0x3C9  /* R/W, RAM Date port */
#define   TVIA3CEINDEX  0x3CE  /* R/W */
#define   TVIA3CFDATA   0x3CF  /* R/W */
#define   TVIA3D4INDEX  0x3D4  /* R/W */
#define   TVIA3D5DATA   0x3D5  /* R/W */
#define   TVIA3C4INDEX  0x3C4  /* R/W */
#define   TVIA3C5DATA   0x3C5  /* R/W */


/*Access TV Enacode registers*/
void   WriteTVReg(WORD_ index, WORD_ wData);
WORD_  ReadTVReg(WORD_ index);

/*Access Audio registers*/
void   WriteAudioRegB(WORD_ index, BYTE_ bData);
BYTE_  ReadAudioRegB(WORD_ index);
void   WriteAudioRegW(WORD_ index, WORD_ wData);
WORD_  ReadAudioRegW(WORD_ index);
void   WriteAudioRegD(WORD_ index, DWORD_ dwData);
DWORD_ ReadAudioRegD(WORD_ index);

/*
  Access 2D Engine registers.
  Because 2D part is performance sensetive, we do not use function calls,
  instead, use Macros.
*/
  #define Write2DRegB(index,value) *(volatile BYTE_  far_ *)(bpTVAudio2DRegBase+index)=(BYTE_)value
  #define Write2DRegW(index,value) *(volatile WORD_  far_ *)(bpTVAudio2DRegBase+index)=(WORD_)value
  #define Write2DRegD(index,value) *(volatile DWORD_ far_ *)(bpTVAudio2DRegBase+index)=(DWORD_)value
  #define Read2DRegB(index)        *(volatile BYTE_  far_ *)(bpTVAudio2DRegBase+index)
  #define Read2DRegW(index)        *(volatile WORD_  far_ *)(bpTVAudio2DRegBase+index)
  #define Read2DRegD(index)        *(volatile DWORD_ far_ *)(bpTVAudio2DRegBase+index)


/*Access generic registers*/
/*=========================================================================
  outp(), inp(), outpw() and inpw() are defined in <conio.h> under Borland C++
  and Micosoft C (Both are DOS version compilers).

  We redefined them as tviaoutp(), tviainp(), tviaoutpw() and tviainpw() for the
  cases of:
    1) Using memory mapped I/O.
    2) No standard library available.
  ========================================================================*/
void   tviaoutp(WORD_ wPortAddr, BYTE_ bData);
BYTE_  tviainp(WORD_ wPortAddr);
void   tviaoutpw(WORD_ wPortAddr, WORD_ wData);
WORD_  tviainpw(WORD_ wPortAddr);


#if LINEAR_MEMORY
/* ==================== !!! Important Notice !!! ==========================
  The two routines below are x86 assembly code only and must be taken care
  of by SDK porting developers.
  ========================================================================*/
void   asm_outp(WORD_ wPortAddr, BYTE_ bData);
BYTE_  asm_inp(WORD_ wPortAddr);

DWORD_ GetChipPhysAddr(void);
void   EnableLinearAddr(void);
void   DisableLinearAddr(void);
#endif

#ifdef __cplusplus
}
#endif
#endif  /* __TVIA_IGSIOMEM_H */
