/*---------------------------------------------------------------------------
   tvia_overlay.c

   This utility pushes the output of the em84xx decoder to the CyberPro chip.

   Written by HAUS Software :)
  ---------------------------------------------------------------------------*/

#include "global.h"   // global macros
#include "igsiomem.h" // low level h/w programming
#include "utility.h"  // TVIA utility library header file
#include "overlay.h"  // Overlay SDK library header file
#include "capture.h"  // Capture SDK library header file
#include "synclock.h"

#include <stdlib.h>
#include <stdio.h>    // needed only for declaration of printf()
#include <getopt.h>
#include <sys/perm.h>

#define APPLICATION_NAME "tvia_overlay"
#define VERSION "0.1"

#define MAX_WIDTH  1920
#define MAX_HEIGHT 1080

#define STD        PAL
#define FORMAT     CCIR656

int srcWidth=720;
int srcHeight=576;

int destWidth=720;
int destHeight=576;

int bitclipWidth=0;
int bitclipHeight=0;

int shrinkX=0;  // Offset in Pixel welches das Bild horizontal "zusammenstaucht" 
int shrinkY=0;  // Offset in Pixel welches das Bild vertikal "zusammenstaucht" 

int offsetX=0;  // Offset zum horizontalen verschieben des Bildes (nach rechts) (24)
int offsetY=0;  // Offset zum vertikalen verschieben des Bildes (nach unten) (xx)

int bDoubleBuf = FALSE;
int overlayMode = WEAVEMODE;
int bBlank = FALSE;
int bSynclock = FALSE;
int bVerbose = FALSE;

int tvMode = PAL720x576;

#define LINEAR_MEMORY 1
/* note:  This function only works with 5000 RevD
 When enable double-buffer feature, be sure:
 1. Do not use BOBMODE when call Tvia_SetOverlayScale(), use
    WEAVEMODE.
 2. Divide capture pitch width by 2 when call Tvia_SetCapDstAddr()
 3. When call Tvia_SetOverlayScale(), double Overlay vertical DDA
    ratio.
 4. When call Tvia_SetOverlayWindow(WORD_ wLeft, WORD_ wTop,
    WORD_ wRight, WORD_ wBottom), make sure wTop to be an even
    number*/

#if LINEAR_MEMORY
   #include "mapmem.h"  /*initialization of memory mapping*/
#endif


void Usage()
{
   printf("\nUsage : %s [args]\n", APPLICATION_NAME);
   printf(" Valid arguments:\n");
   printf("  -q, --off:            disable Overscan\n" );
   printf("  -b, --bobmode:        use BOBMODE (default: WEAVEMODE)\n" );
   printf("  -w, --destwidth:      destination width (720)\n" );
   printf("  -h, --destheight:     destination height (576)\n" );
   printf("  -x, --offsetx:        horizontal offset (0)\n" );
   printf("  -y, --offsety:        vertical offset (0)\n" );
   printf("  -X, --shrinkx:        horizontal shrink (0)\n" );
   printf("  -Y, --shrinky:        vertical shrink (0)\n" );
   printf("  -B, --bitclipwidth:   horizontal cut (0)\n" );
   printf("  -C, --bitclipheight:  vertical cut (0)\n" );
   printf("  -W, --sourcewidth:    source width (720)\n" );
   printf("  -H, --sourceheight:   source height (576)\n" );
   printf("  -d, --doublebuffer:   use doubleBuffer (for LCD TV)\n" );
   printf("  -o, --blank:          clear output\n" );
   printf("  -s, --synclock:       use synclock function\n" );
   printf("      [0=640x480; 1=720x540; 2=768x576; 3=720x576]\n" );
   printf("  -v, --verbose:        be verbose\n" );
   return;
}

void DisableOverlay()
{
#if LINEAR_MEMORY
   MapTVIAMemory(); /* ohne --> segfault*/  /*to map physical memory to virtual (linear) memory*/
#endif
   Tvia_CaptureOff();  
   Tvia_EnableDoubleBuffer(0);
   Tvia_CaptureCleanUp();  
   Tvia_SelectOverlayIndex(OVERLAY1);
   Tvia_OverlayOff();
   Tvia_SelectOverlayIndex(OVERLAY2);
   Tvia_OverlayOff();
   Tvia_OverlayCleanUp();
   if( bSynclock )
      Tvia_CleanupSyncLock(); //synclock cleanup

   return;
}

void EnableOverlay()
{
   DWORD_ dwOnScreenSize;

   SetEnvironmentVariables();

   dwOnScreenSize = (DWORD_)GetCurrentScreenMapSize(ON_SCREEN) * 1024L;

   // Overlay1 will display whole video at the upper-right corner 
   Tvia_SelectOverlayIndex(OVERLAY1);  // call this function at the very top

   //destWidth += (bitclipWidth *1); //Zielvideo dementsprechend vergroessern (4x bei 1024x768);
   //destHeight += (bitclipHeight*1);

   if( bDoubleBuf == TRUE )          // For 5000 RevD Doublebuffer, capture destination
      destWidth = (destWidth /8) * 8;  // has to be 8 pixels boundary

   Tvia_SetOverlayFormat(YUV422); // always call this function before call Tvia_SetSrcAddr(),
                                  // and most video decoder send out data in YUV422 format
   Tvia_SetOverlayMode(WINDOWKEY);
   Tvia_SetOverlaySrcAddr(dwOnScreenSize, 0, 0, srcWidth, srcWidth);
   Tvia_SetOverlayWindow( offsetX+(shrinkX/2), offsetY+(shrinkY/2), 
                          destWidth+offsetX-(shrinkX/2), destHeight+offsetY-(shrinkY/2)); //zielwindow koordinaten

   if( bDoubleBuf == TRUE )
      Tvia_SetOverlayScale(overlayMode, srcWidth, destWidth, srcHeight, destHeight*2);
   else
      Tvia_SetOverlayScale(overlayMode, srcWidth, destWidth, srcHeight, destHeight); //scale (zoom in)

   Tvia_ChangeOverlayFIFO();
   Tvia_OverlayOn();

   // program capture engine
   Tvia_SelectCaptureIndex(PORTA); // (PORT A for Activy3xx) Select video port at the very beginning
   Tvia_InitCapture(CCIR656);      // Video decoder send out CCIR656 data, so initialize ours accordingly

   // Do a little bit clipping for incoming external video to get rid of garbage video.
   // incoming video width is 712 - 8 = 704video field height is 241 - 1 = 240
   Tvia_SetCapSrcWindow(bitclipWidth, srcWidth + bitclipWidth, bitclipHeight, srcHeight/2 + bitclipHeight);  

   if( bDoubleBuf == TRUE ) {
      Tvia_SetCapDstAddr(dwOnScreenSize, 0, 0, srcWidth/2);
      Tvia_SetBackBuffer_addr( dwOnScreenSize + (DWORD_)destWidth * (DWORD_)destHeight);
      Tvia_EnableDoubleBuffer(1);
   }
   else {
      // we don't want to screw up onscreen graphics data when we turn off capture, so we set
      // destination address right after graphics data block.And set pitch width to capture destination width
      Tvia_SetCapDstAddr(dwOnScreenSize, 0, 0, srcWidth); 
   }
   Tvia_SetCaptureScale( srcWidth, destWidth - shrinkX, srcHeight/2, destHeight - shrinkY, INTERLACED); //scale
   Tvia_InvertFieldPolarity(0);
   Tvia_CaptureOn();

   // following code is to set capture ending address in case some video decoder output is unstable
   // (for instance, some TV channels) and data would be written beyond expected boundary, after setting
   // the ending addr, video data will be written between starting addr(set by Tvia_SetCapDstAddr()) 
   // and this ending addr. Note: times 2L because video data is two bytes per pixel
   Tvia_SetCapSafeGuardAddr(dwOnScreenSize + (DWORD_)destWidth * (DWORD_)destHeight * 2L);

// test - hintergrund "schwaerzen" ! ?? (bei VGA ist obere Zeile von Freevo blau zu sehen bei Mode 3 und das flimmert ...)
// erstmal so, bis was besseres gefunden ;)
// nur bei 800x600 und 1024x768 noetig, weil hier Bild vom ersten bis zum letzten Pixel angezeigt wird (kein overscan)
   if( bBlank )
      FillRect(0, 0, destWidth, destHeight, 0); 

   if( bSynclock ) {
      //synclock start
      Tvia_DoSyncLock(PORTA, STD, tvMode, FORMAT);
      Tvia_SetTVColor();
      //synclock end
   }

   return;
}  /*end of main()*/


int main(int argc, char *argv[])
{
   static struct option long_options[] = {
      { "off",          no_argument, NULL, 'q'},
      { "bobmode",      no_argument, NULL, 'b'},
      { "destwidth",    required_argument, NULL, 'w'},
      { "destheight",   required_argument, NULL, 'h'},
      { "offsetx",      required_argument, NULL, 'x'},
      { "offsety",      required_argument, NULL, 'y'},
      { "shrinkx",      required_argument, NULL, 'X'},
      { "shrinky",      required_argument, NULL, 'Y'},
      { "sourcewidth",  required_argument, NULL, 'W'},
      { "sourceheight", required_argument, NULL, 'H'},
      { "bitclipwidth", required_argument, NULL, 'B'},
      { "bitclipheight",required_argument, NULL, 'C'},
      { "doublebuffer", no_argument, NULL, 'd'},
      { "blank",        no_argument, NULL, 'o'},
      { "verbose",      no_argument, NULL, 'v'},
      { "synclock",     required_argument, NULL, 's'},
      { "help",         no_argument, NULL, '?'},
      { 0,0,0,0}
   };

   int c, n;
   int bErr = FALSE;
   int bOverscanOff = FALSE;

   printf( "%s v%s\n", APPLICATION_NAME, VERSION );

   iopl(3);

   while( (c = getopt_long(argc, argv, "qbdov?w:W:h:H:x:X:y:Y:B:C:s:", long_options, NULL)) != -1 ) {
      if( optarg )
         n = atoi(optarg);
      else
         n = -9999;

      switch( c ) {
         case 'q':
            bOverscanOff = TRUE;
            break;
         case 'b':
            overlayMode = BOBMODE;
            break;
         case 'd':
            bDoubleBuf = TRUE;
            break;
         case 'o':
            bBlank = TRUE;
            break;
         case 'w':
            if( n >= 0 && n <= MAX_WIDTH )
               destWidth = n;
            else
               bErr = TRUE;
            break;
         case 'h':
            if( n >= 0 && n <= MAX_HEIGHT )
               destHeight = n;
            else
               bErr = TRUE;
            break;
         case 'W':
            if( n >= 0 && n <= MAX_WIDTH )
               srcWidth = n;
            else
               bErr = TRUE;
            break;
         case 'H':
            if( n >= 0 && n <= MAX_HEIGHT )
               srcHeight = n;
            else
               bErr = TRUE;
            break;
         case 'x':
            if( n >= -MAX_WIDTH && n <= MAX_WIDTH )
               offsetX = n;
            else
               bErr = TRUE;
            break;
         case 'y':
            if( n >= -MAX_HEIGHT && n <= MAX_HEIGHT )
               offsetY = n;
            else
               bErr = TRUE;
            break;
         case 'X':
            if( n >= -MAX_WIDTH && n <= MAX_WIDTH )
               shrinkX = n;
            else
               bErr = TRUE;
            break;
         case 'Y':
            if( n >= -MAX_HEIGHT && n <= MAX_HEIGHT )
               shrinkY = n;
            else
               bErr = TRUE;
            break;
         case 'B':
            if( n >= -MAX_WIDTH && n <= MAX_WIDTH )
               bitclipWidth = n;
            else
               bErr = TRUE;
            break;
         case 'C':
            if( n >= -MAX_HEIGHT && n <= MAX_HEIGHT )
               bitclipHeight = n;
            else
               bErr = TRUE;
            break;
         case 'v':
            bVerbose = TRUE;
            break;
         case 's':
            if( n >= 0 && n <= 3 )
               tvMode = n;

            bSynclock = TRUE;
            break;
         case '?':
         default:
            bErr = TRUE;
            break;
      }
   }
   if( bErr == TRUE ) {
      Usage();
      exit(1);
   }
   
   if( bVerbose == TRUE ) {
      if( bOverscanOff == FALSE ) {
         printf( "Overlaymode:  %d\n", overlayMode );
         printf( "DestWidth:    %d\n", destWidth );
         printf( "DestHeight:   %d\n", destHeight );
         printf( "Offset x:     %d\n", offsetX );
         printf( "Offset y:     %d\n", offsetY );
         printf( "Shrink x:     %d\n", shrinkX );
         printf( "Shrink y:     %d\n", shrinkY );
         printf( "BitClWidth:   %d\n", bitclipWidth );
         printf( "BitClHeight:  %d\n", bitclipHeight );
         printf( "SourceWidth:  %d\n", srcWidth );
         printf( "SourceHeight: %d\n", srcHeight );
         printf( "DoubleBuffer: %d\n", bDoubleBuf );
         printf( "Blank Output: %d\n", bBlank );
         printf( "Synclock:     %d - %d\n", bSynclock, tvMode );
      }
      else
         printf( "Overlay mode stopped\n" );
   }
   DisableOverlay();
   if( bOverscanOff == FALSE )
      EnableOverlay();

   exit(0);
}
